<?php
namespace Zeedhi\DevOps_Entities\SyncEngineDataSource;

use Doctrine\ORM\EntityManager;
use Doctrine\ORM\ORMInvalidArgumentException;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\Kernel;

class ManagerImpl extends Manager\Doctrine\ManagerImpl implements Manager {

	/** @var \Zeedhi\Framework\Kernel */
	protected $kernel;
	/** @var SequenceImpl */
	protected $syncEngine;
	/** @var array[] */
	protected $originalRows;

	/**
	 * Construct...
	 *
	 * @param EntityManager                 $entityManager
	 * @param SequenceImpl                  $syncEngine
	 * @param Kernel                        $kernel
	 * @param Manager\Doctrine\NameProvider $nameProvider
	 */
	public function __construct(
		EntityManager $entityManager,
		SequenceImpl $syncEngine,
		Kernel $kernel,
		Manager\Doctrine\NameProvider $nameProvider,
		ParameterBag $parameterBag
	) {
		$this->syncEngine = $syncEngine;
		$this->kernel = $kernel;
		parent::__construct($entityManager, $nameProvider, $parameterBag);
	}

	/**
	 * {@inheritdoc}
	 */
	public function persist(DataSet $dataSet) {
		$this->setDataSourceConfigByDataSet($dataSet);
		$syncDataSet = $this->synchronizeDataSet($dataSet);
        $doctrinePersistedRows = $this->doctrinePersist($syncDataSet);
        return $this->syncEngine->unSyncPkRows($doctrinePersistedRows, $this->dataSourceConfig, $dataSet->getRows());
	}

	/**
	 * {@inheritdoc}
	 */
	public function delete(DataSet $dataSet) {
		$this->setDataSourceConfigByDataSet($dataSet);
		$syncDataSet = $this->synchronizeDataSet($dataSet);
        $doctrineDeletedRows = $this->doctrineDelete($syncDataSet);
        return $this->syncEngine->unSyncPkRows($doctrineDeletedRows, $this->dataSourceConfig, $dataSet->getRows());
	}

	/**
	 * @return mixed
	 */
	protected function getUserId() {
		return $this->kernel->getRequest()->getUserId();
	}

	/**
	 * @param DataSet $dataSet
	 *
	 * @return DataSet
	 */
	protected function synchronizeDataSet(DataSet $dataSet) {
		$this->originalRows = $dataSet->getRows();
		$syncRows = $this->syncEngine->synchronizeRows($this->dataSourceConfig, $this->originalRows, $this->getUserId());
		$syncDataSet = new DataSet($dataSet->getDataSourceName(), $syncRows);
		return $syncDataSet;
	}

	/**
	 * @param DataSet $dataSet
	 *
	 * @return int
	 * @throws \Exception
	 */
	protected function doctrinePersist(DataSet $dataSet) {
		$persistedRows = array();
		foreach ($dataSet->getRows() as $key => $row) {
			$this->beforePersistRowEvent->trigger(array($row, $key));
			$entity = $this->persistRow($row);
			$this->postStoreId($key, $entity);
			$this->afterPersistRowEvent->trigger(array($row, $entity, $key));
			$persistedRows[$key] = $this->getPrimaryKeyValueFromRow($row);
		}

		$this->flushWithTransactionControl();
		return $persistedRows;
	}

	protected function doctrineDelete(DataSet $dataSet) {
		$deletedRows = array();
		foreach ($dataSet->getRows() as $key => $row) {
			// New rows doesn't need to be deleted, since their do not exist.
			if ($row['__is_new'] === false) {
				try {
					$this->beforeDeleteRowEvent->trigger(array($row, $key));
					$entity = $this->deleteRow($row);
					$this->afterDeleteRowEvent->trigger(array($row, $entity, $key));
					$deletedRows[$key] = $this->getPrimaryKeyValueFromRow($row);
				} catch (ORMInvalidArgumentException $e) {
					// When row already deleted, not returned id.
				}
			}
		}

		$this->flushWithTransactionControl();
		return $deletedRows;
	}

	/**
	 * @param $rowKey
	 * @param $entity
	 */
	protected function postStoreId($rowKey, $entity) {
		$originalRow = $this->originalRows[$rowKey];
		$this->syncEngine->postStoreId($this->dataSourceConfig, $originalRow, $this->getUserId(), $entity->getId());
	}

    /**
     * @param FilterCriteria $filterCriteria
     * @return FilterCriteria
     */
    protected function syncFilterCriteria(FilterCriteria $filterCriteria) {
        $config = $this->nameProvider->getDataSourceByName($filterCriteria->getDataSourceName());
        return $this->syncEngine->syncFilterCriteria($filterCriteria, $config, $this->getUserId());
    }

    /**
     * @param DataSet $dataSet
     *
     * @return array with unsynchronized rows.
     */
    protected function unSynchronizeRows($dataSet) {
        $dataSourceName = $dataSet->getDataSourceName();
        $configuration = $this->nameProvider->getDataSourceByName($dataSourceName);
        return $this->syncEngine->unSynchronizeRows($configuration, $dataSet->getRows(), $this->getUserId());
    }

    public function findBy(FilterCriteria $filterCriteria) {
        $syncFilterCriteria = $this->syncFilterCriteria($filterCriteria);
        $dataSet = parent::findBy($syncFilterCriteria);
        $unSyncRows = $this->unSynchronizeRows($dataSet);
        return new DataSet($dataSet->getDataSourceName(), $unSyncRows);
    }
} 
